#include "particlesystem.h"
#include "particlemanager.h"
#include "particleobject.h"

void CMMParticleSystem::CleanUp()
{
	for( int i = 0; i < particlenum; ++i )
	{
		SAFE_RELEASE( particles[i] );
	};
	SAFE_DELETE_ARRAY( particles );
	particlenum = 0;
	particlesize = 0.0f;
	initparticle = NULL;
	updateparticle = NULL;
	isparticlealive = NULL;
	SAFE_RELEASE( texture );
	SAFE_RELEASE( particle );
	animated = false;
};

void CMMParticleSystem::Reset( LPDIRECT3DDEVICE9 DEVICE,
							   int PARTICLENUM, float PARTICLESIZE,
							   LPInitParticle INITPARTICLE,
							   LPUpdateParticle UPDATEPARTICLE,
							   LPIsParticleAlive ISPARTICLEALIVE,
							   CMMTexture* TEXTURE, bool ANIMATED )
{
	CleanUp();
	particlenum = PARTICLENUM;
	particlesize = PARTICLESIZE;
	initparticle = INITPARTICLE;
	updateparticle = UPDATEPARTICLE;
	isparticlealive = ISPARTICLEALIVE;
	particles = new CMMParticle*[particlenum];
	D3DXCreateSprite( DEVICE, &particle );
	for( int i = 0; i < particlenum; ++i )
	{
		particles[i] = Particles::ParticleManager->CreateParticleObject();
		initparticle( this, i, true );
	};
	texture = TEXTURE;
	if( texture ) texture->AddRef();
	animated = ANIMATED;
};

void CMMParticleSystem::Update( int UPDATES )
{
	for( int i = 0; i < particlenum; ++i )
	{
		updateparticle( this, i );
		if( !isparticlealive( this, i ) ) initparticle( this, i, false );
	};
};

CMMParticle* CMMParticleSystem::GetParticle( int PARTICLEINDEX )
{
	if( PARTICLEINDEX >=0 && PARTICLEINDEX < particlenum) return particles[PARTICLEINDEX];
	else return NULL;
};

void CMMParticleSystem::SetTexture( CMMTexture* TEXTURE, bool ANIMATED )
{
	SAFE_RELEASE( texture );
	texture = TEXTURE;
	if( texture ) texture->AddRef();
	animated = ANIMATED;
};

void CMMParticleSystem::Draw( LPDIRECT3DDEVICE9 DEVICE )
{
	particle->Begin( D3DXSPRITE_ALPHABLEND );
	D3DSURFACE_DESC pDesc;
	texture->GetTexture()->GetLevelDesc( 0, &pDesc );
	D3DXMATRIXA16 scale;
	D3DXMatrixScaling( &scale, 1.0f / pDesc.Width * particlesize,
		1.0f / pDesc.Height * particlesize, 1.0f );
	for( int i = 0; i < particlenum; ++i )
	{
		particle->SetTransform( &scale );
		D3DXVECTOR3 pos = particles[i]->GetPosition();
		pos.x *= pDesc.Width / particlesize;
		pos.y *= pDesc.Height / particlesize;
		particle->Draw( texture->GetTexture(), NULL, NULL, &pos,
			particles[i]->GetColor() );
	};
	particle->End();
};