#include <windows.h>
#include <string>
#include <d3dx9.h>
#include "texturemanager.h"
#include "textureobject.h"

void CMMTexture::CleanUp()
{
	SAFE_RELEASE( texture );
	filename = L"";
};

HRESULT CMMTexture::LoadTexture( LPDIRECT3DDEVICE9 DEVICE, LPWSTR FILENAME )
{
	filename = FILENAME;
	SAFE_RELEASE( texture );
	return D3DXCreateTextureFromFile( DEVICE, FILENAME, &texture );
};

LPDIRECT3DTEXTURE9 CMMTexture::GetTexture()
{
	return texture;
};

std::wstring CMMTexture::GetTextureFileName()
{
	return filename;
};

void CMMMultiTexture::CleanUp()
{
	for( int i = 0; i < texturenum; ++i )
	{
		SAFE_RELEASE( textures[i] );
	};
	SAFE_DELETE_ARRAY( textures );
	filename = L"";
	texture = NULL;
	texturenum = 0;
	textureindex = -1;
};

HRESULT CMMMultiTexture::LoadTexture( LPDIRECT3DDEVICE9 DEVICE, LPWSTR FILENAME )
{
	if( texturenum == 0 ) return E_FAIL;
	HRESULT hr = S_OK;
	SAFE_RELEASE( textures[textureindex] );
	CMMTexture* Obj = Textures::TextureManager->FindTexture( FILENAME );
	if( Obj )
	{
		Obj->AddRef();
	}
	else
	{
		Obj = Textures::TextureManager->CreateTextureObject( TEXTURE_SIMPLE );
		hr = Obj->LoadTexture( DEVICE, FILENAME );
	};
	textures[textureindex] = Obj;
	SetTextureIndex( textureindex );
	return hr;
};

void CMMMultiTexture::SetTextureNumber( int TEXTURENUM )
{
	CleanUp();
	texturenum = TEXTURENUM;
	if( texturenum <= 0 )
	{
		texturenum = 0;
		return;
	};
	textures = new CMMTexture*[texturenum];
	for( int i = 0; i < texturenum; ++i )
	{
		textures[i] = NULL;
	};
	SetTextureIndex( 0 );
};

int CMMMultiTexture::GetTextureNumber()
{
	return texturenum;
};

void CMMMultiTexture::SetTextureIndex( int TEXTUREINDEX )
{
	textureindex = TEXTUREINDEX;
	if( textureindex < 0 ) textureindex = 0;
	else if ( textureindex >= texturenum ) textureindex = texturenum - 1;
	if( textures[textureindex] )
	{
		texture = textures[textureindex]->GetTexture();
		filename = textures[textureindex]->GetTextureFileName();
	}
	else
	{
		texture = NULL;
		filename = L"";
	};
};

int CMMMultiTexture::GetTextureIndex()
{
	return textureindex;
};

void CMMMultiTexture::Next( bool CIRCLE )
{
	int i = textureindex + 1;
	if( i >= texturenum && CIRCLE ) i = 0;
	SetTextureIndex( i );
};

void CMMMultiTexture::Prev( bool CIRCLE )
{
	int i = textureindex - 1;
	if( i < 0 && CIRCLE ) i = texturenum - 1;
	SetTextureIndex( i );
};